@@ -23,7 +23,7 @@ use constant +{
     COMPRESS_SAVINGS => 0.20,
 };
 
-our $VERSION = '0.00022';
+our $VERSION = '0.00023';
 
 sub new {
     my $class = shift;
@@ -116,7 +116,7 @@ sub _on_tcp_connect {
     delete $self->{_is_connecting}->{$server}; # thanks, buddy
     if (! $fh) {
         # connect failed
-        warn("failed to connect to $server");
+        warn("failed to connect to $server [$!]");
 
         if ($self->{auto_reconnect} > $self->{_connect_attempts}->{ $server }++) {
             # XXX this watcher holds a reference to $self, which means
@@ -177,6 +177,7 @@ sub connect {
 
     return if $self->{_is_connecting} || $self->{_is_connected};
     $self->disconnect();
+    delete $self->{_is_disconnecting};
 
     $self->{_is_connecting} = {};
     $self->{_active_servers} = [];
@@ -277,7 +278,7 @@ sub _push_queue {
 sub _drain_queue {
     my $self = shift;
     if (! $self->{_is_connected}) {
-        if ($self->{_is_connecting}) {
+        if ($self->{_is_connecting} or $self->{_is_disconnecting}) {
             return;
         }
         $self->connect;
@@ -320,6 +321,7 @@ sub disconnect {
     delete $self->{_is_draining};
 
     $self->{_server_handles} = {};
+    $self->{_is_disconnecting}++;
 }
 
 sub DESTROY {
