@@ -1,11 +1,14 @@
 package Cache::Memcached::AnyEvent::Selector::Traditional;
 use strict;
-use Carp qw(croak);
-use String::CRC32 qw(crc32);
+use Carp ();
+use String::CRC32 ();
 
 sub new {
     my $class = shift;
-    bless { @_, buckets => [], bucketcount => 0 }, $class;
+    my $self = bless { @_, buckets => [], bucketcount => 0 }, $class;
+
+    $self->set_servers($self->{memcached}->servers);
+    return $self;
 }
 
 sub set_servers {
@@ -19,7 +22,7 @@ sub set_servers {
 # for object definition's sake, it's good to make this a method
 # but for efficiency, this ->hashkey call is just stupid
 sub hashkey {
-    return (crc32($_[1]) >> 16) & 0x7fff;
+    return (String::CRC32::crc32($_[1]) >> 16) & 0x7fff;
 }
 
 sub get_handle {
@@ -47,7 +50,7 @@ sub get_handle {
         }
     }
 
-    croak "Could not find a suitable handle for key $key";
+    Carp::croak("Could not find a suitable handle for key $key");
 }
 
 1;
