@@ -19,7 +19,11 @@ sub run {
     my $warn_called = 0;
     local $SIG{__WARN__} = sub {
 #        like( $_[0], qr/^failed to connect to $bogus_server/);
-        $warn_called++;
+        if ( $_[0] =~ /^failed to connect to $bogus_server/ ) {
+            $warn_called++;
+        } else {
+            warn @_;
+        }
     };
 
     foreach my $i (1..50) {
@@ -43,4 +47,4 @@ sub run {
     done_testing;
 }
 
-1;
\ No newline at end of file
+1;
